@@ -322,9 +322,9 @@ prepare_lvmdbusd() {
 		# Setup the python path so we can run
 		export PYTHONPATH=$abs_top_builddir/daemons
 	else
-		daemon="$(which lvmdbusd || :)"
+		daemon=$(which lvmdbusd || :)
 	fi
-	[[ -n $daemon && -x $daemon ]] || skip "The daemon is missing"
+	[[ -n "$daemon" && -x "$daemon" ]] || skip "The daemon is missing"
 	which python3 >/dev/null || skip "Missing python3"
 
 	python3 -c "import pyudev, dbus, gi.repository" || skip "Missing python modules"
@@ -405,23 +405,28 @@ teardown_devs_prefixed() {
 	fi
 
 	# Remove devices, start with closed (sorted by open count)
-	local remfail=no
+	# Run 'dmsetup remove' in parallel
 	local need_udev_wait=0
+	rm -f REMOVE_FAILED
+	#local listdevs=( $(dm_info name,open --sort open,name | grep "$prefix.*:0") )
+	#dmsetup remove --deferred ${listdevs[@]%%:0} || touch REMOVE_FAILED
+
 	init_udev_transaction
-	for dm in $(dm_info name --sort open | grep "$prefix"); do
-		dmsetup remove "$dm" &>/dev/null || remfail=yes
+	for dm in $(dm_info name --sort open,name | grep "$prefix"); do
+		dmsetup remove "$dm" &>/dev/null || touch REMOVE_FAILED &
 		need_udev_wait=1
 	done
+	wait
 	finish_udev_transaction
 	test $need_udev_wait -eq 0 || udev_wait
 
-	if test $remfail = yes; then
+	if test -f REMOVE_FAILED; then
 		local num_devs
 		local num_remaining_devs=999
 		while num_devs=$(dm_table | grep "$prefix" | wc -l) && \
 		    test $num_devs -lt $num_remaining_devs -a $num_devs -ne 0; do
 			test "$stray" -eq 0 || echo "Removing $num_devs stray mapped devices with names beginning with $prefix: "
-                        # HACK: sort also by minors - so we try to close 'possibly later' created device first
+			# HACK: sort also by minors - so we try to close 'possibly later' created device first
 			for dm in $(dm_info name --sort open,-minor | grep "$prefix") ; do
 				dmsetup remove -f "$dm" || true
 			done
@@ -520,6 +525,11 @@ teardown() {
 
 	if test -f TESTNAME ; then
 
+	if test ! -f SKIP_THIS_TEST ; then
+		# Evaluate left devices only for non-skipped tests
+		TEST_LEAKED_DEVICES=$(dmsetup table | grep "$PREFIX" | grep -v "${PREFIX}pv") || true
+	fi
+
 	kill_tagged_processes
 
 	if test -n "$LVM_TEST_LVMLOCKD_TEST" ; then
@@ -567,6 +577,12 @@ teardown() {
 
 	fi
 
+	test -z "$TEST_LEAKED_DEVICES" || {
+		echo "Unexpected devices left dm table:"
+		echo "$TEST_LEAKED_DEVICES"
+		return 1
+	}
+
 	test -n "$TESTDIR" && {
 		cd "$TESTOLDPWD"
 		rm -rf "$TESTDIR" || echo BLA
@@ -783,6 +799,7 @@ prepare_devs() {
 
 	local size=$(($devsize*2048)) # sectors
 	local count=0
+	rm -f CREATE_FAILED
 	init_udev_transaction
 	for i in $(seq 1 $n); do
 		local name="${PREFIX}$pvname$i"
@@ -790,17 +807,19 @@ prepare_devs() {
 		DEVICES[$count]=$dev
 		count=$(( $count + 1 ))
 		echo 0 $size linear "$BACKING_DEV" $((($i-1)*$size + $shift)) > "$name.table"
-		if not dmsetup create -u "TEST-$name" "$name" "$name.table" &&
-		   test -n "$LVM_TEST_BACKING_DEVICE";
-		then # maybe the backing device is too small for this test
-		    LVM_TEST_BACKING_DEVICE=
-		    rm -f BACKING_DEV
-		    prepare_devs "$@"
-		    return $?
-		fi
+		dmsetup create -u "TEST-$name" "$name" "$name.table" || touch CREATE_FAILED &
+		test -f CREATE_FAILED && break;
 	done
+	wait
 	finish_udev_transaction
 
+	if test -f CREATE_FAILED -a -n "$LVM_TEST_BACKING_DEVICE"; then
+		LVM_TEST_BACKING_DEVICE=
+		rm -f BACKING_DEV CREATE_FAILED
+		prepare_devs "$@"
+		return $?
+	fi
+
 	# non-ephemeral devices need to be cleared between tests
 	test -f LOOP || for d in ${DEVICES[@]}; do
 		blkdiscard "$d" 2>/dev/null || true
@@ -822,9 +841,11 @@ prepare_devs() {
 #	( IFS=$'\n'; echo "${DEVICES[*]}" ) >DEVICES
 	echo "ok"
 
-	for dev in "${DEVICES[@]}"; do
-		notify_lvmetad "$dev"
-	done
+	if test -e LOCAL_LVMETAD; then
+		for dev in "${DEVICES[@]}"; do
+			notify_lvmetad "$dev"
+		done
+	fi
 }
 
 
@@ -845,7 +866,7 @@ common_dev_() {
 		else
 			test -z "${offsets[@]}" && offsets="0:"
 		fi ;;
-	error)  offsets=${@:3}
+	error|zero)  offsets=${@:3}
 		test -z "${offsets[@]}" && offsets="0:" ;;
 	esac
 
@@ -872,8 +893,8 @@ common_dev_() {
 		case "$tgtype" in
 		delay)
 			echo "$from $len delay $pvdev $(($pos + $offset)) $read_ms $pvdev $(($pos + $offset)) $write_ms" ;;
-		error)
-			echo "$from $len error" ;;
+		error|zero)
+			echo "$from $len $tgtype" ;;
 		esac
 		pos=$(($pos + $len))
 	done > "$name.devtable"
@@ -964,15 +985,51 @@ enable_dev() {
 	done
 }
 
+# Once there is $name.devtable
+# this is a quick way to restore to this table entry
+restore_from_devtable() {
+	local dev
+	local silent
+
+	if test "$1" = "--silent"; then
+	    silent=1
+	    shift
+	fi
+
+	rm -f debug.log strace.log
+	init_udev_transaction
+	for dev in "$@"; do
+		local name=$(echo "$dev" | sed -e 's,.*/,,')
+		dmsetup load "$name" "$name.devtable"
+		dmsetup resume "$name"
+	done
+	finish_udev_transaction
+
+	test -n "$silent" || for dev in "$@"; do
+		notify_lvmetad "$dev"
+	done
+}
+
 #
 # Convert device to device with errors
 # Takes the list of pairs of error segment from:len
-# Original device table is replace with multiple lines
+# Combination with zero or delay is unsupported
+# Original device table is replaced with multiple lines
 # i.e.  error_dev "$dev1" 8:32 96:8
 error_dev() {
 	common_dev_ error "$@"
 }
 
+#
+# Convert existing device to a device with zero segments
+# Takes the list of pairs of zero segment from:len
+# Combination with error or delay is unsupported
+# Original device table is replaced with multiple lines
+# i.e.  zero_dev "$dev1" 8:32 96:8
+zero_dev() {
+	common_dev_ zero "$@"
+}
+
 backup_dev() {
 	local dev
 
@@ -1063,7 +1120,7 @@ generate_config() {
             cat > "$config_values" <<-EOF
 activation/checks = 1
 activation/monitoring = 0
-activation/polling_interval = 0
+activation/polling_interval = 1
 activation/retry_deactivation = 1
 activation/snapshot_autoextend_percent = 50
 activation/snapshot_autoextend_threshold = 50
@@ -1156,22 +1213,24 @@ EOF
 lvmconf() {
 	unset profile_name
 	generate_config "$@"
-	mv -f CONFIG etc/lvm.conf
+	mv -f CONFIG "$LVM_SYSTEM_DIR/lvm.conf"
 }
 
 profileconf() {
+	local pdir="$LVM_SYSTEM_DIR/profile"
 	profile_name="$1"
 	shift
 	generate_config "$@"
-	mkdir -p etc/profile
-	mv -f "PROFILE_$profile_name" "etc/profile/$profile_name.profile"
+	mkdir -p $pdir
+	mv -f "PROFILE_$profile_name" "$pdir/$profile_name.profile"
 }
 
 prepare_profiles() {
-	mkdir -p etc/profile
+	local pdir="$LVM_SYSTEM_DIR/profile"
+	mkdir -p $pdir
 	for profile_name in $@; do
 		test -L "lib/$profile_name.profile" || skip
-		cp "lib/$profile_name.profile" "etc/profile/$profile_name.profile"
+		cp "lib/$profile_name.profile" "$pdir/$profile_name.profile"
 	done
 }
 
@@ -1260,7 +1319,7 @@ udev_wait() {
 wait_for_sync() {
 	local i
 	for i in {1..100} ; do
-		check in_sync $1 $2 && return
+		check in_sync $1 $2 $3 && return
 		sleep .2
 	done
 
@@ -1377,10 +1436,10 @@ have_raid() {
 }
 
 have_raid4 () {
-	local r=1
+	local r=0
 
-	have_raid 1 8 0 && r=0
-	have_raid 1 9 1 && r=1
+	have_raid 1 8 0 && r=1
+	have_raid 1 9 1 && r=0
 
 	return $r
 }
@@ -1480,6 +1539,27 @@ wait_pvmove_lv_ready() {
 	fi
 }
 
+# Holds device open with sleep which automatically expires after given timeout
+# Prints  PID of running holding sleep process in background
+hold_device_open() {
+	local vgname=$1
+	local lvname=$2
+	local sec=${3:-20} # default 20sec
+
+	sleep $sec < "$DM_DEV_DIR/$vgname/$lvname" >/dev/null 2>&1 &
+	SLEEP_PID=$!
+	# wait till device is openned
+	for i in $(seq 1 50) ; do
+		if test "$(dmsetup info --noheadings -c -o open $vgname-$lvname)" -ne 0 ; then
+			echo "$SLEEP_PID"
+			return
+		fi
+		sleep .1
+	done
+
+	die "$vgname-$lvname expected to be openned, but it's not!"
+}
+
 # return total memory size in kB units
 total_mem() {
 	while IFS=":" read -r a b ; do
